
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */


#ifndef __TRACKER_ENUMS_TYPES_H__
#define __TRACKER_ENUMS_TYPES_H__

#include <glib-object.h>
#include "tracker-enums.h"

G_BEGIN_DECLS

/* enumerations from "tracker-date-time.h" */
GType tracker_date_error_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_DATE_ERROR (tracker_date_error_get_type ())

/* enumerations from "tracker-dbus.h" */
GType tracker_dbus_events_type_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_DBUS_EVENTS_TYPE (tracker_dbus_events_type_get_type ())
GType tracker_dbus_error_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_DBUS_ERROR (tracker_dbus_error_get_type ())

/* enumerations from "tracker-enums.h" */
GType tracker_verbosity_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_VERBOSITY (tracker_verbosity_get_type ())
GType tracker_sched_idle_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_SCHED_IDLE (tracker_sched_idle_get_type ())
GType tracker_serialization_format_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_SERIALIZATION_FORMAT (tracker_serialization_format_get_type ())

/* enumerations from "tracker-locale.h" */
GType tracker_locale_id_get_type (void) G_GNUC_CONST;
#define TRACKER_TYPE_LOCALE_ID (tracker_locale_id_get_type ())
G_END_DECLS

#endif /* __TRACKER_ENUMS_TYPES_H__ */

/* Generated data ends here */

